@@ -1,12 +1,7 @@
-import functools
-import json
-
 from django.db import transaction
 from django.db.backends.base.features import BaseDatabaseFeatures
 from django.db.backends.signals import connection_created
-from django.db.backends.sqlite3.base import none_guard
 from django.db.utils import OperationalError
-from django.utils.version import PY38
 
 
 class DatabaseFeatures(BaseDatabaseFeatures):
@@ -18,6 +13,11 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     has_native_json_field = False
     # Does the backend use PostgreSQL-style JSON operators like '->'?
     has_json_operators = False
+    # Does the backend support __contains and __contained_by lookups for a JSONField?
+    supports_json_field_contains = True
+    # Does value__d__contains={'f': 'g'} (without a list around the dict) match
+    # {'d': [{'f': 'g'}]}?
+    json_key_contains_list_matching_requires_list = False
 
 
 class MySQLFeatures(DatabaseFeatures):
@@ -29,11 +29,13 @@ class MySQLFeatures(DatabaseFeatures):
 
 class OracleFeatures(DatabaseFeatures):
     supports_primitives_in_json_field = False
+    supports_json_field_contains = False
 
 
 class PostgresFeatures(DatabaseFeatures):
     has_native_json_field = True
     has_json_operators = True
+    json_key_contains_list_matching_requires_list = True
 
 
 class SQLiteFeatures(DatabaseFeatures):
@@ -45,6 +47,8 @@ class SQLiteFeatures(DatabaseFeatures):
             return False
         return True
 
+    supports_json_field_contains = False
+
 
 feature_classes = {
     "mysql": MySQLFeatures,
@@ -53,13 +57,7 @@ feature_classes = {
     "sqlite": SQLiteFeatures,
 }
 
-
-feature_names = [
-    "supports_json_field",
-    "supports_primitives_in_json_field",
-    "has_native_json_field",
-    "has_json_operators",
-]
+feature_names = set(dir(DatabaseFeatures)) - set(dir(BaseDatabaseFeatures))
 
 
 def extend_features(connection, **kwargs):
@@ -70,26 +68,5 @@ def extend_features(connection, **kwargs):
         setattr(connection.features, name, value)
 
 
-@none_guard
-def _sqlite_json_contains(haystack, needle):
-    target, candidate = json.loads(haystack), json.loads(needle)
-    if isinstance(target, dict) and isinstance(candidate, dict):
-        return target.items() >= candidate.items()
-    return target == candidate
-
-
-def extend_sqlite(connection, **kwargs):
-    if connection.vendor != "sqlite":
-        return
-    if PY38:
-        create_deterministic_function = functools.partial(
-            connection.connection.create_function, deterministic=True,
-        )
-    else:
-        create_deterministic_function = connection.connection.create_function
-    create_deterministic_function("JSON_CONTAINS", 2, _sqlite_json_contains)
-
-
 def connect_signal_receivers():
     connection_created.connect(extend_features)
-    connection_created.connect(extend_sqlite)
